# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
###########################################################################

"""
Agent zephir pour le test des tunnels
"""

from twisted.python import log
from twisted.internet.utils import getProcessOutput
from zephir.monitor.agentmanager.agent import Agent, RRDAgent
from zephir.monitor.agentmanager.data import HTMLData, TableData
from zephir.monitor.agentmanager.util import status_to_img
from zephir.monitor.agentmanager import status

import os

def parse_ipsec_status(response):
    """parses ipsec status info
    response format : 
    ['18:', '"C-amon1-0-sphynx"', 'STATE_QUICK_R2', '(IPsec', 'SA', 'established);', 'EVENT_SA_REPLACE', 'in', '8219s;', 'newest', 'IPSEC;', 'eroute', 'owner']
    ['18:', '"C-amon1-0-sphynx"', 'esp.9c0c70fd@192.168.230.59', '(5016', 'bytes,', '519s', 'ago)', 'esp.c4cacef1@192.168.230.49', '(2772', 'bytes,', '519s', 'ago);', 'tunnel']
    ['C-amon1-0-sphynx":', '172.16.36.0/24===192.168.230.49[@sphynxNG.agriates.lan]---192.168.230.254...192.168.230.254---192.168.230.59[@amon1]===10.21.10.0/24;', 'erouted;', 'eroute', 'owner:', '#18'] *** si owner = #0 --> tunnel down ***
    """
    response = response.split('\n')
    tunnels = []
    total=up=down=0
    for line in response:
        if not line[4:].startswith('#') and line.count('routed') > 0:
            data = line[4:].split(';')
            name, adresses = data[0].split(':')
            src = adresses[:adresses.index('---')]
            dst = adresses[adresses.rindex('---')+3:]
            if 'erouted' in [line.strip() for line in data] or 'unrouted' in [line.strip() for line in data]:
                conn_id = line[line.index('owner:')+6:].strip()
                if conn_id != '#0':
                    # l'eroute existe (infos à récupérer)
                    status = 'On'
                    up += 1
                else:
                    status = 'Off'
                    down += 1
                total += 1
                tunnels.append({'name':name.replace('"',''), 'src':src, 'dst':dst, 'status':status})
    return tunnels, total, up, down

class RvpAmon(Agent):

    def __init__(self, name,
                 **params):
        Agent.__init__(self, name, **params)
        self.status = status.Unknown()
        self.table = TableData([
            ('name', "Identifiant", {'align':'left'}, None),
            ('status', "Etat", {'align':'center'}, status_to_img), 
            ('src', "Source", {'align':'left'}, None),
            ('dst', "Destination", {'align':'left'}, None),
            ])
        self.data = [self.table]
        self.measure_data = {}

    def measure(self):
        self.status = status.OK()
        cmd = "/usr/sbin/ipsec"
        update = getProcessOutput(cmd, ['status'])
        return update.addCallbacks(self.callback_tunnels, self.errback_tunnels)

    def callback_tunnels(self,response):
        tunnels, total, up, down = parse_ipsec_status(response)
        if down > 0:
            self.status=status.Error()
        for data in tunnels:
            self.measure_data[data['name']] = (data['src'], data['dst'], data['status'])
        self.measure_data['ok'] = up
        self.measure_data['bad'] = down
        return {'statistics':tunnels}

    def errback_tunnels(self, err):
        self.status = status.Error("Erreur d'execution : ipsec status")
        return {'statistics':None}

    def write_data(self):
        Agent.write_data(self)
        if self.last_measure is not None:
            self.table.table_data = self.last_measure.value['statistics']
            
    def check_status(self):
        return self.status

class RvpSphynx(RRDAgent):

    pourcentok = 100

    def measure(self):
        self.status = status.OK()
        cmd = "/usr/sbin/ipsec"
        update = getProcessOutput(cmd, ['status'])
        return update.addCallbacks(self.callback_tunnels, self.errback_tunnels)

    def callback_tunnels(self,response):
        tunnels, total, up, down = parse_ipsec_status(response)
        self.measure_data['ok'] = up
        self.measure_data['bad'] = down
        if total != 0:
            self.pourcentok = (100*up)/total
        else:
            self.pourcentok = 100
        return {'ok':up,'bad':down,'total':total,'pourcentok':self.pourcentok}

    def errback_tunnels(self, err):
        self.status = status.Error("Erreur d'execution : ipsec status")
        res = None
        self.pourcentok = 100
        return {'statistics':res}

    def check_status(self):
        if self.pourcentok < 75:
            return status.Error("plus de 25% de tunnels en erreur")
        if self.pourcentok < 95:
            return status.Warn("plus de 5% de tunnels en erreur")
        return status.OK()

