# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
###########################################################################

"""
Agent zephir pour la surveillance de services via tcpcheck
"""

from twisted.python import log
from twisted.internet import defer
from twisted.internet.utils import getProcessOutput, getProcessValue
from os import system

from zephir.monitor.agentmanager.agent import Agent
from zephir.monitor.agentmanager.data import HTMLData, TableData
from zephir.monitor.agentmanager.util import status_to_img, boolean_to_onoff
from zephir.monitor.agentmanager import status

TCPCHECK = '/usr/bin/tcpcheck'

def compare_hostports(hp1, hp2):
    h1, p1 = hp1.split(':')
    h2, p2 = hp2.split(':')
    result = cmp(h1, h2) # alpha comparison on hosts
    if result != 0:
        return result
    else: # numeric comparison on ports
        return cmp(int(p1), int(p2))

class TCPServices(Agent):

    def __init__(self, name,
                 tcp_services, # dict {"host:port": "comment"}
                 module,
                 **params):
        Agent.__init__(self, name, **params)
        self.tcp_services = tcp_services
        self.module = module
        self.table = TableData([
            ('status', "état", {'align':'center'}, status_to_img), 
            ('description', "Description", {'align':'left'}, None),
            ('host', "Hôte", {'align':'left'}, None),
            ('port', "Port", {'align':'left'}, None) ])
        self.data = [self.table]


    def measure(self):
        tcpcheck = getProcessOutput(TCPCHECK,
                                    args = ['1'] + self.tcp_services.keys(),
                                    env = {'LC_ALL': 'C'})
        tcpcheck.addCallback(self.measure_process)
        return tcpcheck
    
    def measure_process(self, tcpcheck_result):
        lines = tcpcheck_result.splitlines()
        hostports_status = [l.split(' ', 1) for l in lines]
        hostports_status.sort(lambda (hp1, s1), (hp2, s2):
                              compare_hostports(hp1, hp2))
        for hostport, status in hostports_status:
            status = boolean_to_onoff(status.endswith('alive'))
            mesures=[{ 'description': self.tcp_services[hostport],
                       'host': hostport.split(':')[0],
                       'port': hostport.split(':')[1],
                       'status': status }]
            self.measure_data['status'] = status
        return {'services': mesures}

    def write_data(self):
        Agent.write_data(self)
        if self.last_measure is not None:
            self.table.table_data = self.last_measure.value['services']

    def check_status(self):
        """remonte une erreur si un des services est tombé"""
        if self.last_measure is not None:
            for service in self.last_measure.value['services']:
                if service['status'] != 'On':
                    return status.Error()
        else:
            # pas de mesure connue
            return status.Unknown()
        return status.OK()


class PIDService(Agent):

    def __init__(self, name,
                 tcp_services, # dict {"daemon": "comment"}
                 module,
                 **params):
        Agent.__init__(self, name, **params)
        self.tcp_services = tcp_services
        self.module = module
        self.table = TableData([
            ('status', "état", {'align':'center'}, status_to_img), 
            ('description', "Description", {'align':'left'}, None),
            ('host', "Hôte", {'align':'left'}, None),
            ('port', "Port", {'align':'left'}, None) ])
        self.data = [self.table]

    def measure(self):

        res = system('/bin/pidof %s >/dev/null'%self.tcp_services.keys()[0])
        status = boolean_to_onoff(res==0)
        mesures=[{ 'description': self.tcp_services.values()[0],
                   'host': 'localhost',
                   'port': '',
                   'status': status }]
        self.measure_data['status'] = status
        return {'services': mesures}

    def write_data(self):
        Agent.write_data(self)
        if self.last_measure is not None:
            self.table.table_data = self.last_measure.value['services']

    def check_status(self):
        """remonte une erreur si un des services est tombé"""
        if self.last_measure is not None:
            for service in self.last_measure.value['services']:
                if service['status'] != 'On':
                    return status.Error()
        else:
            # pas de mesure connue
            return status.Unknown()
        return status.OK()


class VPNService(Agent):

    def __init__(self, name,
                 tcp_services, # dict {"host:port": "comment"}
                 module,
                 **params):
        Agent.__init__(self, name, **params)
        self.tcp_services = tcp_services
        self.module = module
        self.table = TableData([
            ('status', "Etat", {'align':'center'}, status_to_img), 
            ('description', "Description", {'align':'left'}, None),
            ('host', "Hôte", {'align':'left'}, None),
            ('port', "Port", {'align':'left'}, None) ])
        self.data = [self.table]


    def measure(self):
        cmd = "/usr/sbin/ipsec"
        vpncheck = getProcessOutput(cmd,
                                    args=['status'],
                                    env = {'LC_ALL': 'C'})
        vpncheck.addCallbacks(self.measure_vpn,self.service_error)
        return vpncheck
        
    def service_error(self,data):
        """status a remonté une erreur
        """
        log.msg("erreur status : %s" % str(data))
        status = 'Off'
        mesures=[{ 'description': "Service Réseau Virtuel Privé (IpSec)",
                   'host': 'localhost',
                   'port': '',
                   'status': status }]
        return {'services': mesures}

    def measure_vpn(self, data):
        status = 'Off'
        # si not running apparait dans les premières lignes de la sortie, ipsec est arreté
        data = data.split('\n')
        end = 5
        if len(data) < 5:
            end = len(data)
        first_lines = "".join(data[0:end])
        if first_lines.count('not running') == 0:
            status = 'On'
         
        mesures=[{ 'description': "Service Réseau Virtuel Privé (IpSec)",
                   'host': 'localhost',
                   'port': '',
                   'status': status }]
        return {'services': mesures}

    def write_data(self):
        Agent.write_data(self)
        if self.last_measure is not None:
            self.table.table_data = self.last_measure.value['services']

    def check_status(self):
        """remonte une erreur si un des services est tombé"""
        if self.last_measure is not None:
            for service in self.last_measure.value['services']:
                if service['status'] != 'On':
                    return status.Error()
        else:
            # pas de mesure connue
            return status.Unknown()
        return status.OK()

